/***************************************************************************
 *   Copyright (C) 2010 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ombattery.h"
#include <QDebug>
#include <QApplication>
#include <QDomDocument>

OMBattery::OMBattery(QWidget *parent)
 : QWidget(parent)
{

	MainLayout = new QVBoxLayout();

	properties << QObject::tr("capacity:") << QObject::tr("charging:") << QObject::tr("charge now:") << QObject::tr("current:") << QObject::tr("voltage:") << QObject::tr("temperature:") << QObject::tr("time to full:") << QObject::tr("time to empty:") << QObject::tr("technology:") << QObject::tr("health:");

	NAString = QObject::tr("n/a");

	IconBasePath = "/usr/share/pixmaps/openmoko-panel-plugin-qt/";

	
	InfoBox = new OMInfoBox( QObject::tr("battery"), this);
	MainLayout->addWidget(InfoBox);
	int n;
	for ( n = 0; n < 10; n++) {

		InfoBox->addPanel(properties.at(n), QObject::tr("unknown") );

		}


	DialogButtons = new OMDialogButtons(OMFlags::QuitButton | OMFlags::CloseButton, this);
	MainLayout->addWidget( DialogButtons );
	connect(DialogButtons, SIGNAL(clicked(QFlags< OMFlags :: Flag >)), this, SLOT(DialogButtonClicked(QFlags< OMFlags :: Flag >)));

	setLayout( MainLayout );

	SysTrayIcon = new QSystemTrayIcon( this );
	connect(SysTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));



	connectToBattery();
	update();

	SysTrayIcon->show();
	

	connect(BatteryInterface, SIGNAL(PowerStatus ( QString )), this, SLOT(PowerStatuschanged ( QString )));
	connect(BatteryInterface, SIGNAL(Capacity ( int )), this, SLOT(Capacitychanged ( int )));

	Timer = new QTimer( this );
	Timer->setInterval( 10000 );
	connect(Timer,SIGNAL(timeout()),this,SLOT(update()));
	Timer->start();

	

}


OMBattery::~OMBattery()
{
}




void OMBattery::connectToBattery()
{


/*	QDBusInterface PowerSupplyInterface("org.freesmartphone.odeviced", "/org/freesmartphone/Device/PowerSupply", "org.freedesktop.DBus.Introspectable", QDBusConnection::systemBus(), this);
	
	if (!PowerSupplyInterface.isValid()) { qDebug() << "Cannont connect to powersupply interface. Bye.."; qApp->quit();}

	QDBusReply <QString> xml = PowerSupplyInterface.call("Introspect");

//	qDebug() << xml;

	QDomDocument data;
	data.setContent(xml);
	QDomElement node = data.documentElement();
      	QDomElement child = node.firstChildElement();

	bool batteryfound = false;

	while (!child.isNull() && !batteryfound) {
		if (child.tagName() == "node") {
			
			qDebug() << "probing /org/freesmartphone/Device/PowerSupply/" + child.attribute("name");

			BatteryInterface = new QDBusInterface("org.freesmartphone.odeviced", "/org/freesmartphone/Device/PowerSupply/" + child.attribute("name"), "org.freesmartphone.Device.PowerSupply", QDBusConnection::systemBus(), this);
			
			QDBusReply< QMap < QString,QVariant> > reply = BatteryInterface->call("GetInfo");
			if (reply.value().value("name").toString() == "battery") batteryfound=true;
			}
		child = child.nextSiblingElement();
		}

	if (batteryfound) qDebug() << "battery found at " << BatteryInterface->path() ;
	else qDebug() << "battery no found";
*/

BatteryInterface = new QDBusInterface( "org.freesmartphone.odeviced", OMBatteryPath(), "org.freesmartphone.Device.PowerSupply", QDBusConnection::systemBus(), this );
 	

}


void OMBattery::update()
{
    	if ( BatteryInterface->isValid() ) {
		
		QDBusReply< QMap < QString,QVariant> > reply = BatteryInterface->call("GetInfo");

		QMapIterator<QString, QVariant> i(reply);
		while (i.hasNext()) {
     			i.next();

			QString key = i.key();
			if ( key == "capacity") {
				InfoBox->changePanelValue(properties.at(0), i.value().toString() + " %");
				BatteryCharge = i.value().toInt();
				}
			if ( key == "status") {
				InfoBox->changePanelValue(properties.at(1), i.value().toString());
				if (i.value().toString() == "Charging") BatteryCharging = true; else BatteryCharging = false;
				}
			if ( key == "charge_now") InfoBox->changePanelValue(properties.at(2), QString::number( i.value().toInt() / 1000 ) + " mAh");
			if ( key == "current_now") InfoBox->changePanelValue(properties.at(3), QString::number( i.value().toInt() / 1000 ) + " mAh");
			if ( key == "voltage_now") InfoBox->changePanelValue(properties.at(4), QString::number( i.value().toInt() / 1000 ) + " mV");
			if ( key == "temp") InfoBox->changePanelValue(properties.at(5), QString::number( i.value().toInt() / 10 ) + QString::fromUtf8(" °C"));

			if ( key == "time_to_full_now") InfoBox->changePanelValue(properties.at(6), secToHMS( i.value().toInt()) );
			if ( key == "time_to_empty_now") InfoBox->changePanelValue(properties.at(7),secToHMS( i.value().toInt()) );

			if ( key == "technology") InfoBox->changePanelValue(properties.at(8), i.value().toString());
			if ( key == "health") InfoBox->changePanelValue(properties.at(9), i.value().toString());
				

			}

		setBatteryIcon();
		}
		
	else { 
		qDebug() << "cannot connect to Battery";
		connectToBattery();
		}
		 


}


QString OMBattery::secToHMS( int sec )
{
    	int h,m,s;
	
	if ( sec == 3932100 ) return NAString;
	else {
		h = sec / 3600;
		sec = sec / 3600;
		m = sec / 60;
		s = sec & 60;

		QString hms( QString::number( h ) + "h, " + QString::number( m ) + "min, " + QString::number( s ) + "s");
		
		return hms;
		}
}


void OMBattery::setBatteryIcon()
{
    	
	int p = BatteryCharge / 10;
	
	QString path = IconBasePath + "battery_";

	if (p < 10) path = path + "0";

	path = path + QString::number( p ) + "0";

	if (BatteryCharging) path = path + "_charging_500";

	path = path + ".png";
//	qDebug() << path;	
	QIcon baticon(path);
 	SysTrayIcon->setIcon( baticon );
	

}


void OMBattery::iconActivated(QSystemTrayIcon::ActivationReason)
{
	update();
	show();
}


void OMBattery::PowerStatuschanged ( QString status )
{
    update();
}


void OMBattery::Capacitychanged ( int cap )
{
    update();
}


void OMBattery::DialogButtonClicked(QFlags<OMFlags::Flag> flags)
{
    if ( flags.testFlag(OMFlags::QuitButton) ) qApp->quit();
    if ( flags.testFlag(OMFlags::CloseButton) ) hide();
}
